import argparse
from dataclasses import dataclass
from typing import cast


@dataclass
class Args:
    """
    Data class representing command-line arguments.
    """

    is_song: bool
    is_debug: bool
    no_tag: bool
    lyrics: bool
    dump_json: bool
    query: str
    dir: str


def parse_args() -> Args:
    """
    Parses command-line arguments.

    :returns: Parsed arguments as an instance of Args.
    """
    # Initialize parser
    parser = argparse.ArgumentParser(
        prog="muclic", description="A CLI for downloading music", exit_on_error=True
    )

    # Add arguments
    _ = parser.add_argument(
        "query", type=str, help="Album/song name", nargs="*", default=""
    )
    _ = parser.add_argument(
        "-d", "--dir", type=str, help="Specify output direcory", default="~/Music"
    )

    # Add switches
    _ = parser.add_argument(
        "-s",
        "--song",
        help="Download a single song",
        action="store_true",
        default=False,
    )
    _ = parser.add_argument(
        "-T", "--no-tag", help="Don't tag songs", action="store_true", default=False
    )
    _ = parser.add_argument(
        "-l", "--lyrics", help="Download lyrics", action="store_true", default=False
    )
    _ = parser.add_argument(
        "--dump-json",
        help="Dump a single json file with info on downloaded items. For developement use only",
        action="store_true",
        default=False,
    )
    _ = parser.add_argument(
        "--debug", help="Set log level to debug", action="store_true", default=False
    )

    # Read arguments from command line and cast them to Args class
    args = parser.parse_args()

    args.query = " ".join(cast(str, args.query))

    return Args(
        is_song=cast(bool, args.song),
        is_debug=cast(bool, args.debug),
        no_tag=cast(bool, args.no_tag),
        lyrics=cast(bool, args.lyrics),
        dump_json=cast(bool, args.dump_json),
        query=args.query,
        dir=cast(str, args.dir),
    )



#!/usr/bin/python
# pyright: strict
# pyright: reportUnnecessaryTypeIgnoreComment=false
import os
import sys

import muclic.logging as logs
from muclic.app import App
from muclic.helper_types import SearchResult


def main() -> None:
    try:
        app = App()
        ytlogger: logs.YtDLLogger = logs.setup_logging(app.args.is_debug)
        search_results: list[SearchResult] = app.search()
        user_choices: list[int] = app.get_user_choices(search_results)

        app.create_media_items(user_choices, search_results)
        app.download_items(ytlogger)
        app.download_lyrics()
        temp_files = app.tag_items()

        # Cleanup
        for file in temp_files:
            os.remove(file)
    except KeyboardInterrupt:
        sys.exit(0)


if __name__ == "__main__":
    try:
        main()
    except KeyboardInterrupt:
        sys.exit(0)



from typing import TypedDict


class Thumbnail(TypedDict):
    """
    Represents a thumbnail image with its width and URL pointing to it.
    """

    width: int
    url: str


class SearchResult(TypedDict):
    """
    Result of a youtubemusic search independently of the filter.
    Has more fields, but we care only about those.
    """

    title: str
    artists: list[dict[str, str]]


class SongSearchResult(SearchResult):
    """
    Result of a youtubemusic search with a 'songs' filter.
    Has more fields, but we care only about those.
    """

    videoId: str
    album: dict[str, str]


class AlbumSearchResult(SearchResult):
    """
    Result of a youtubemusic search with a 'albums' filter.
    Has more fields, but we care only about those.
    """

    browseId: str
    thumbnails: list[Thumbnail]


class YTAlbumData(TypedDict):
    """
    Data received from downloading album from YTMusic.
    Has more fields, but we care only about those.
    """

    audioPlaylistId: str


class SongInfo(TypedDict):
    """
    Info dumped by YoutubeDL.
    Has more fields, but only these are needed for tagging.
    """

    release_year: int | None
    artist: str | list[str]
    album: str
    track: str
    genre: str
    track_number: int
    n_entries: int
    playlist_index: int
    thumbnails: list[Thumbnail]


class AlbumInfo(TypedDict):
    """
    Info dumped by YoutubeDL.
    Has more fields, but only these are needed for tagging.
    """

    thumbnails: list[Thumbnail]
    entries: list[SongInfo]



from abc import ABC, abstractmethod
from dataclasses import dataclass

from muclic.helper_types import AlbumInfo, SongInfo
from muclic.logging import YtDLLogger


@dataclass
class MediaItem(ABC):
    """
    Abstract base class for media items.
    """

    title: str
    artist: str
    path: str
    url: str
    cover: str | None
    info: SongInfo | AlbumInfo | None

    @abstractmethod
    def download(self, ytlogger: YtDLLogger) -> None: ...

    @abstractmethod
    def download_lyrics(self) -> None: ...

    @abstractmethod
    def tag(self) -> None: ...

    def get_cover(self, temp_files: list[str]) -> None: ...  # pyright: ignore[reportUnusedParameter]
